/*
** libdebug.h for Project in /home/phil/Travail/Thales/hookconnector/code/nfhcctl/src/lib/debug
**
** Made by thierr_p
** Login   <philippe.THIERRY@reseau-libre.net>
**
** Started on  ven 22 ao 2008 16:58:22 CET thierr_p
** Last update lun 17 aoû 2009 09:36:18 CEST t0090690
*/
/*
** Copyright (C) 2009 - Philippe THIERRY
** This program is free software; you can redistribute it and/or
** modify it under the terms of the GNU General Public License
** as published by the Free Software Foundation; either version 2
** of the License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

/*
** CAUTION: This file is the libdebug interface. It should be used by whatever
** software using this lib.
*/



#ifndef LIBDEBUG_H_
# define LIBDEBUG_H_

#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#include <inttypes.h>
#include <ctype.h>
#include <sys/types.h>
#ifndef _AIX
#include <stdint.h>
#endif
#include <string.h>
#include <signal.h>
#include <time.h>
#include <syslog.h>

#include "libutils.h"
#include "libmemtrace.h"

/*!
** \file libdebug.h is the interface of the debug library.
**
** This library implements different capacities of debugging, warning,
** erroring and panic mechanisms.
*/

/*
** all return value for lib_common call....
*/
typedef enum
{
  RET_OK,     /**< call to libcommon service is ok */
  RET_ERROR,  /**< call to libcommon service is error */
  RET_MAX     /**< max value for sizing this object */
} e_ret_value;

#define CM_MAX_PREFIX_SIZE		   30
#define CM_MAX_SIZE_BUFFER_SYSLOG 512
#define CM_DEBUG_LAST_MESSAGE     "LAST_MESSAGE"
#define CM_DEBUG_FIRST_MESSAGE    "FIRST_MESSAGE"
#define CM_DEFAULT_PREFIX         "libdebug_prefix"

/*!
** @brief This enum describe the different severities of debugging.
**
** It is compliant to RFC3164 (table 2). Of course there is a hierarchy
** in this definition, so it is the responsabilty of user to use this enumerate with 
** correct severity according expected behaviour.
*/
typedef enum
  {
	DEBUG_PANIC,     /**< system is unusable */
	DEBUG_ALERT,     /**< action must be taken immediatly*/
	DEBUG_CRIT,      /**< critical conditions */
	DEBUG_ERROR,     /**< error conditions*/
	DEBUG_WARNING,   /**< warning conditions */
	DEBUG_NOTICE,    /**< normal but signifiant notice */
	DEBUG_INFO,      /**< simple informational */
	DEBUG_DEBUG,     /**< debug-severity message */
	DEBUG_MAX        /**< max debug value. also used for invalid value */
  } e_severity;

/*
** @brief This enum describes the different type of 'channel' on which
** it is possible to send output debug message.
*/
typedef enum
  {
	DBG_CONNECTOR_STDERR,  /**< all output data flow is redirected to STDERR device  */
	DBG_CONNECTOR_SYSLOG,  /**< all output data flow is redirected to syslog local API, (depending on implementation)  */
	DBG_CONNECTOR_NONE,    /**< all output data flow is redirected is never send (equivalent to /dev/null) */
	DBG_CONNECTOR_MAX
  } e_connector;

/*!
** @brief get the current severity
**
** When calling it, return the current running severity in use
**
** @param nothing
**
** @return enumerate of e_severity type
**
*/
e_severity
cm_debug_get_severity(void);


/*!
** \def Specify the return value of a panic exit
*/
#define	DEBUG_KILL	127

/*!
** @brief get the current connector type.
**
** When calling it, return the current running connector type
**
** @param noting
**
** @return enumerate of e_connector type
**
*/
e_connector
cm_debug_get_connector (void);

/*!
** @brief get the severity string translate from enumerate
**
** When calling it, return the ASCII string corresponding to the input enumerate
** If input enumerate is invalid, then output string is forced to invalid RFC3164 value
**
** @param enumerate of e_severity type
**
** @return string corresponding to input enumerate
**
*/
const char*
cm_debug_get_severity_string (e_severity severity);

/*!
** @brief get the current running prefix
**
** When calling it, return the ASCII string corresponding to the running prefix
**
** @param noting
**
** @return string corresponding to running prefix
**
*/
const char*
cm_debug_get_prefix (void);


/*!
** @brief get the current running counter
**
** When calling it, return current counter
**
** @param noting
**
** @return current running counter
**
*/
uint32_t
cm_debug_get_count (void);

/*!
** The DBG_TRACE macro is used for debugging information
**
** This macro automaticaly support syslog/stderr logging facilities, depending
** on the cm_debug_init() call.
** Debug
**
** The explicit cast is due to printf incapacity to manage time_t types.
*/

#define DBG_TRACE(severity, ...)					    \
  if (DEBUG_MAX > severity)						    \
  {									    \
    e_connector current_connector;					    \
    e_severity current_severity;					    \
    /*									    \
    ** evaluate current connector and severity as defined during	    \
    ** cm_debug_init sequence						    \
    */									    \
    current_connector = cm_debug_get_connector();			    \
    current_severity = cm_debug_get_severity();				    \
    if (current_connector == DBG_CONNECTOR_STDERR)			    \
    {									    \
      if ((int) severity > (int) current_severity)			    \
      {									    \
        fprintf(stderr,                                                     \
                "%s %s %010d %08lld %04d %s %s ",                           \
                cm_debug_get_prefix(),                                      \
                cm_debug_get_severity_string(severity),                     \
                cm_debug_get_count(),                                       \
                (long long int)time(NULL),                                  \
                __LINE__,                                                   \
                __FILE__,                                                   \
                __func__);						    \
        fprintf(stderr, __VA_ARGS__);					    \
        fprintf(stderr, "\n");						    \
      }									    \
    }									    \
    else								    \
    {									    \
      if (current_connector == DBG_CONNECTOR_SYSLOG)			    \
      {									    \
        char dbgtrace_string[CM_MAX_SIZE_BUFFER_SYSLOG];		    \
        uint32_t	dbgtrace_len = 0;				    \
        if ((int) severity > (int) current_severity)			    \
        {								    \
          dbgtrace_len = snprintf(&dbgtrace_string [0],                     \
                                  CM_MAX_SIZE_BUFFER_SYSLOG,                \
                                  "%s %010d %08lld %04d %s %s ",            \
                                  cm_debug_get_severity_string(severity),   \
                                  cm_debug_get_count(),                     \
                                  (long long int)time(NULL),                \
                                  __LINE__,                                 \
                                  __FILE__,                                 \
                                  __func__);				    \
          dbgtrace_len = snprintf(&(dbgtrace_string[dbgtrace_len]),         \
                                  CM_MAX_SIZE_BUFFER_SYSLOG - dbgtrace_len, \
                                  __VA_ARGS__);				    \
          syslog(severity,                                                  \
                 "%s\n",                                                    \
                 &dbgtrace_string [0]);					    \
        }								    \
      }									    \
    }									    \
  }									    \

/*!
** @brief Print a panic and exit.
**
** When calling it, replace LINE with gcc standard __LINE__ value and replace
** FILENAME with gcc standard __file__ value.
**
** If libdebug has been initialized with a sighandler, the sighandler is
** called first before leaving.
**
** @param LINE line of code on which this function is called
** @param FILENAME the corresponding source file on which this function is called
** @param str the string to print
**
*/
void
panic(uint16_t		LINE,
      const char	*FILENAME,
      const char	*str,
      ...);


/*
** @brief cm_debug_set_severity set a local severity according incoming parameter.
**
** If severity is >= DEBUG_MAX, then local initialisation is made to DEBUG_PANIC value
** and returning is done to DEBUG_PANIC.
**
** @return the saved local severity, either incomming severity or DEBUG_PANIC
*/
e_severity
cm_debug_set_severity(e_severity  severity);

/*!
** @brief initialize the debug manager.
**
** This function should be called in order to specify the debug severity and
** the debug output type (stderr or syslog or none), ie the device on which all
** data traffic is send when using later DBG_TRACE fonction. After all internal test
** and all processing, a first trace is sent to the device providing there is no 
** error detected during the treatment. The first message edited to the device use  
** standard DEB_TRACE fonction with following pattern CM_DEBUG_FIRST_MESSAGE.
** The severity used for sending this first trace is DEBUG_INFO.
** 
** @param severity	the debug severity you want (conform to RFC3164), from DEBUG_PANIC to DEBUG_DEBUG .
** If severity is >= DEBUG_MAX, then local initialisation is made to DEBUG_PANIC value
** and returning is done to RET_ERROR. There is a hierarchy in definition, please look at RFC3164
** 
** @param connector 	the debug output you want : stderr or syslog facility or none.
** if connector >= DBG_CONNECTOR_MAX, then local initialisation is made to DBG_CONNECTOR_STDERR value
** and returning is done to RET_ERROR in order to indicate a bad input value
** 
** @param prefix 	the prefix string you would like in each trace, please note that 
** string  must never be longer than CM_MAX_PREFIX_SIZE, otherwise CM_DEFAULT_PREFIX is used
** and returning is done to RET_ERROR in order to indicate a bad input value
** If prefix is a null pointer, then default prefix is use . (CM_DEFAULT_PREFIX) and returning is done to RET_OK
**
** @return RET_OK if init OK, RET_ERROR otherwise
*/
e_ret_value
cm_debug_init(e_severity		severity,
              e_connector		connector,
              const char * const	prefix);


/*!
** @brief cm_debug_release
**
** this fonction edit a last message to the current open device by using 
** standard DEB_TRACE fonction with following pattern CM_DEBUG_LAST_MESSAGE.
** The severity used for sending this last trace is DEBUG_INFO.
** After this action, the current device is closed
**
** @param noting
**
** @return noting
*/
void
cm_debug_release(void);

#endif /* !LIBDEBUG_H_ */
